package ch.uzh.ifi.ddis.fip.esper.test;

/*
 * #%L
 * Eviction for Esper
 * %%
 * Copyright (C) 2013 University of Zurich
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * #L%
 */

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Optional;
import org.testng.annotations.Parameters;

import com.espertech.esper.client.Configuration;
import com.espertech.esper.client.EPServiceProvider;
import com.espertech.esper.client.EPServiceProviderManager;

/**
 * Generic Test for the Esper complex event processing engine.
 * 
 * @author Thomas Scharrenbach
 * @version 0.0.1
 * @since 0.0.1
 * 
 */
public abstract class EsperTest {

	protected static final Logger _log = LoggerFactory
			.getLogger(EsperTest.class);

	//
	//
	//

	private final Configuration _configuration = new Configuration();

	private EPServiceProvider _epService;

	//
	//
	//

	/**
	 * Executed before the code of the {@link #configure(String)} method is
	 * being executed. Note that {@link #getConfiguration()} always returns a
	 * configuration object that is not null.
	 */
	protected abstract void beforeConfigure();

	/**
	 * 
	 * @param esperConfig
	 */
	@BeforeMethod
	@Parameters({ "esper-config" })
	public void configure(@Optional("") String esperConfig) {
		beforeConfigure();

		if (!esperConfig.trim().isEmpty()) {
			try {
				_log.info(
						"Started loading Esper configuration from file {}...",
						esperConfig);
				_configuration.configure(esperConfig);
				_log.info("Finished loading Esper configuration from file {}.",
						esperConfig);
			} catch (Exception e) {
				_log.info("Error loading Esper configuration from file {}!",
						esperConfig, e);
				assert false;
			}
		}
		afterConfigure();

		assert true;
		return;
	}

	/**
	 * Executed after the code of the {@link #configure(String)} method is being
	 * executed.
	 */
	protected abstract void afterConfigure();

	/**
	 * Initializes the service with the default provider.
	 */
	@BeforeMethod(dependsOnMethods = { "configure" })
	public void initEngine() {
		_epService = EPServiceProviderManager
				.getDefaultProvider(_configuration);

	}

	/**
	 * Adds the statements from the provided statement files.
	 * 
	 * @param statementFile
	 */
	@BeforeMethod(dependsOnMethods = { "initEngine" })
	@Parameters({ "statement-files-list" })
	public void addStatements(String statementFilesList) {
		BufferedReader reader = null;

		// TODO write parser...
		for (String statementFile : statementFilesList.split(",")) {
			try {
				_log.info("Started reading statement file {}...", statementFile);
				reader = new BufferedReader(new FileReader(new File(
						statementFile)));
				StringBuffer buffer = new StringBuffer();
				String line = null;

				// read all lines into the buffer.
				while ((line = reader.readLine()) != null) {
					buffer.append(line);
				}
				_epService.getEPAdministrator().createEPL(
						buffer.toString().trim());

				_log.info("Finished reading statement file {}.", statementFile);
			} catch (Exception e) {
				_log.error("Error reading statement file {}!", statementFile, e);
				assert false;
			}
			// close the reader if possible.
			finally {
				if (reader != null) {
					try {
						_log.info("Started closing statement file {}...",
								statementFile);
						reader.close();
						_log.info("Finished closing statement file {}.",
								statementFile);
					} catch (IOException e) {
						_log.error("Error closing statement file {}!",
								statementFile, e);
						assert false;
					}
				}
			}
		}
	}

	//
	//
	//

	public Configuration getConfiguration() {
		return _configuration;
	}

	public EPServiceProvider getEpService() {
		return _epService;
	}

}
